# Generated by roxygen2: do not edit by hand

S3method(forecast,lst_bayesRecon_BUIS)
S3method(forecast,lst_bayesRecon_MixCond)
S3method(forecast,lst_bayesRecon_TDcond)
S3method(forecast,lst_bayesRecon_t)
export(bayesRecon_BUIS)
export(bayesRecon_MixCond)
export(bayesRecon_TDcond)
export(bayesRecon_t)
importFrom(bayesRecon,.check_hierarchical)
importFrom(bayesRecon,.compute_naive_cov)
importFrom(bayesRecon,.core_reconc_BUIS)
importFrom(bayesRecon,.core_reconc_MixCond)
importFrom(bayesRecon,.core_reconc_TDcond)
importFrom(bayesRecon,.core_reconc_t)
importFrom(bayesRecon,.get_HG)
importFrom(bayesRecon,multi_log_score_optimization)
importFrom(bayesRecon,schaferStrimmer_cov)
importFrom(distributional,cdf)
importFrom(distributional,dist_inflated)
importFrom(distributional,dist_sample)
importFrom(distributional,dist_student_t)
importFrom(distributional,dist_truncated)
importFrom(distributional,generate)
importFrom(distributional,support)
importFrom(dplyr,full_join)
importFrom(fabletools,distribution_var)
importFrom(fabletools,forecast)
importFrom(purrr,exec)
importFrom(purrr,list_c)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map_dbl)
importFrom(purrr,map_int)
importFrom(purrr,pmap)
importFrom(purrr,reduce)
importFrom(rlang,abort)
importFrom(rlang,enquo)
importFrom(stats,complete.cases)
importFrom(stats,cov2cor)
importFrom(stats,density)
importFrom(stats,family)
importFrom(stats,frequency)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(tsibble,index_var)
importFrom(vctrs,field)
importFrom(vctrs,vec_c)
importFrom(vctrs,vec_slice)
