data("swiss_tourism", package = "bayesRecon")
saveRDS(swiss_tourism, file = "tests/testthat/data/swiss_tourism.rds")


data("extr_mkt_events", package = "bayesRecon")
saveRDS(extr_mkt_events, file = "tests/testthat/data/extreme_mkt_events.rds")


