% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bayesRecon_TDcond.R
\name{forecast.lst_bayesRecon_TDcond}
\alias{forecast.lst_bayesRecon_TDcond}
\title{forecast.lst_bayesRecon_TDcond}
\usage{
\method{forecast}{lst_bayesRecon_TDcond}(
  object,
  key_data,
  point_forecast = list(.mean = mean),
  new_data = NULL,
  ...
)
}
\arguments{
\item{object}{An object of class \code{lst_bayesRecon_MixCond} containing fitted models.}

\item{key_data}{A keyed data frame from \code{fabletools}.}

\item{point_forecast}{A list of point forecast functions (default: \code{list(.mean = mean)}).}

\item{new_data}{Optional new data for forecasting (not currently used).}

\item{...}{extra parameters to be passed on.}
}
\value{
A fable object with TDcond-reconciled distributions and point forecasts.
}
\description{
Produces probabilistic forecasts reconciled via top-down conditioning.
Upper base forecasts are reconciled via conditioning. The recondiled
upper forecasts are then propagate to bottom via probabilisitc top-down.
}
\keyword{internal}
